#!/usr/bin/env python
"""
mgenerate is no longer included with mtools. Please use
`mgeneratejs <https://www.npmjs.com/package/mgeneratejs>`__ instead.
"""

import sys


def main():
    print("mgenerate is no longer included with mtools. Please use "
          "https://www.npmjs.com/package/mgeneratejs instead.")


if __name__ == '__main__':
    sys.exit(main())
